import TypeUtils from "./utils/type";

import ActionsRunMultiple from "./actions/runMultiple";

import ActionsFormsSubmit from "./actions/forms/submit";
import ActionsFieldsReset from "./actions/fields/reset";
import ActionsFieldsFocus from "./actions/fields/focus";

import ActionsHttpGetV1 from "./actions/http/get";
import ActionsHttpPostV1 from "./actions/http/post";
import ActionsHttpPatchV1 from "./actions/http/patch";
import ActionsHttpDeleteV1 from "./actions/http/delete";

import ActionsDialogsAlert from "./actions/dialogs/alert";
// import ActionsDialogsConfirm from "./actions/dialogs/confirm"
import ActionsDialogsShow from "./actions/dialogs/show";
import ActionsDialogsOpen from "./actions/dialogs/open";
import ActionsDialogsClose from "./actions/dialogs/close";
import ActionsDialogsReload from "./actions/dialogs/reload";
import ActionsDialogsOptions from "./actions/dialogs/options";
import ActionsDialogsNotification from "./actions/dialogs/notification";
import ActionsDialogsOauth from "./actions/dialogs/oauth";

import ActionsSnackbarsAlert from "./actions/snackbars/alert";
// import ActionsSnackbarsConfirm from "./actions/snackbars/confirm"
import ActionsSnackbarsSelect from "./actions/snackbars/select";

import ActionsSheetsSelect from "./actions/sheets/select";

import ActionsWindowsClose from "./actions/windows/close";
import ActionsWindowsCloseAll from "./actions/windows/closeAll";
import ActionsWindowsOpen from "./actions/windows/open";
import ActionsWindowsOpenWeb from "./actions/windows/openWeb";
import ActionsWindowsReload from "./actions/windows/reload";
import ActionsWindowsCloseWithReload from "./actions/windows/closeWithReload";
import ActionsWindowsRefreshState from "./actions/windows/refreshState";
import ActionsWindowsPrint from "./actions/windows/print";

import ActionsPanelsScrollToBottom from "./actions/panels/scrollToBottom";
import ActionsPanelsScrollTo from "./actions/panels/scrollTo";

import ActionsWsPush from "./actions/ws/push";
import ActionsCablesPush from "./actions/cables/push";

import ActionsTimeoutsSet from "./actions/timeouts/set";

import ActionsAuthSaveCsrfToken from "./actions/auth/saveCsrfToken";
import ActionsAuthRestart from "./actions/auth/restart";
import ActionsCreditCard from "./actions/auth/creditCard";

import ActionsAnalyticsLogEvent from "./actions/analytics/logEvent";

import ActionCommandsCopy from "./actions/commands/copy";
import ActionCommandsCustom from "./actions/commands/custom";

// TODO: handle promise action
const ActionToursStart = import("./actions/tours/start");
const ActionToursStop = import("./actions/tours/stop");

import ActionPopoversOpen from "./actions/popovers/open";
import ActionPopoversClose from "./actions/popovers/close";

import ActionComponentsUpdate from "./actions/components/update";
import ActionComponentsFind from "./actions/components/find";

import ActionLabelsSet from "./actions/labels/set";

import ActionListsAppend from "./actions/lists/append";

import { vueApp } from "./store";

const actions = {
  runMultiple: ActionsRunMultiple,

  "forms/submit": ActionsFormsSubmit,
  "fields/reset": ActionsFieldsReset,
  "fields/focus": ActionsFieldsFocus,

  "http/get": ActionsHttpGetV1,
  "http/post": ActionsHttpPostV1,
  "http/patch": ActionsHttpPatchV1,
  "http/delete": ActionsHttpDeleteV1,

  "dialogs/alert": ActionsDialogsAlert,
  // "dialogs/confirm": ActionsDialogsConfirm,
  "dialogs/show": ActionsDialogsShow,
  "dialogs/open": ActionsDialogsOpen,
  "dialogs/close": ActionsDialogsClose,
  "dialogs/reload": ActionsDialogsReload,
  "dialogs/options": ActionsDialogsOptions,
  "dialogs/notification": ActionsDialogsNotification,
  "dialogs/oauth": ActionsDialogsOauth,

  "snackbars/alert": ActionsSnackbarsAlert,
  // "snackbars/confirm": ActionsSnackbarsConfirm,
  "snackbars/select": ActionsSnackbarsSelect,

  "sheets/select": ActionsSheetsSelect,

  "windows/close": ActionsWindowsClose,
  "windows/closeAll": ActionsWindowsCloseAll,
  "windows/open": ActionsWindowsOpen,
  "windows/openWeb": ActionsWindowsOpenWeb,
  "windows/reload": ActionsWindowsReload,
  "windows/closeWithReload": ActionsWindowsCloseWithReload,
  "windows/refreshState": ActionsWindowsRefreshState,
  "windows/print": ActionsWindowsPrint,

  "panels/scrollToBottom": ActionsPanelsScrollToBottom,
  "panels/scrollTo": ActionsPanelsScrollTo,

  "timeouts/set": ActionsTimeoutsSet,

  "ws/push": ActionsWsPush,
  "cables/push": ActionsCablesPush,

  "auth/saveCsrfToken": ActionsAuthSaveCsrfToken,
  "auth/restart": ActionsAuthRestart,
  "auth/creditCard": ActionsCreditCard,

  "analytics/logEvent": ActionsAnalyticsLogEvent,

  "commands/copy": ActionCommandsCopy,
  "commands/custom": ActionCommandsCustom,

  "tours/start": ActionToursStart,
  "tours/stop": ActionToursStop,

  "popovers/open": ActionPopoversOpen,
  "popovers/close": ActionPopoversClose,

  "components/update": ActionComponentsUpdate,
  "components/find": ActionComponentsFind,

  "labels/set": ActionLabelsSet,
  "lists/append": ActionListsAppend
};

const customActions = {};

export default class Action {
  static execute(spec, component, params = {}) {
    if (!TypeUtils.isObject(spec)) {
      return;
    }
    if (!TypeUtils.isObject(component)) {
      console.warn("Action requires a component");
      return;
    }

    const name = spec.action;
    if (TypeUtils.isString(name)) {
      if (name.startsWith("component/")) {
        return this.executeLocal(name, spec, component);
      } else {
        return this.executeGlobal(name, spec, component, params);
      }
    } else {
      console.warn(`Invalid action: "${name}"`);
    }
  }

  static executeLocal(name, spec, component) {
    const methodName = name.replace(/^component\//, "");
    const componentName = GLib.component.vueName(component);
    console.debug(
      `Executing component action on ${componentName}: ${methodName}`
    );
    component[`action_${methodName}`](spec);
  }

  static executeGlobal(name, spec, component, params) {
    return this._executeInternal(name, spec, component, params, actions);
  }

  static _executeInternal(name, spec, component, params, registry) {
    const actionName = name.replace(/-v1$/, "");

    try {
      const action = new registry[actionName]();
      const logDisabled = action.logDisabled && action.logDisabled();
      if (!logDisabled && !spec.silent) {
        console.debug(`Executing "${actionName}"`);
      }
      return action.execute(spec, component, params);
    } catch (e) {
      GLib.settings.errorHandler(e, `Failed executing "${actionName}"`);
      return null;
    }
  }

  static handleResponse(response, component, windowMode = true) {
    // The execution goes in the following order: onResponse, page render, onLoad
    vueApp.temp.analytics = response.analytics;
    GLib.action.execute(response.onResponse, component);
    vueApp.temp.analytics = null;

    if (response.header || response.body || response.footer) {
      Utils.http.forceComponentUpdate(() => {
        const dialog = component.$closest("dialog")
        const updateDialog = windowMode ? false : Utils.type.isObject(dialog)
        if (updateDialog) {
          dialog.updateContent(response);
        } else {
          vueApp.page = response;
        }
      });
    }
  }

  static registerCustom(actionName, action) {
    if (customActions[actionName]) {
      console.error("Command already registered", actionName);
    } else {
      customActions[actionName] = action;
    }
  }

  static executeCustom(name, spec, component, params) {
    return this._executeInternal(name, spec, component, params, customActions);
  }
}

window.Action = Action;
