import { watchEffect } from "vue";
import { glibevent, ctx } from "../../store";

export function useDirtyState() {
  const prompt = "Changes you made have not been saved. Are you sure?";

  const isDirty = () => {
    return ctx().isFormDirty && !ctx().isFormSubmitted && !confirm(prompt);
  };

  const clearDirtyState = () => {
    ctx().isFormDirty = false;
  };

  const updateDirtyState = (val, oldVal, spec) => {
    if (!spec.disableDirtyCheck && val != oldVal && val != spec.value) {
      ctx().isFormDirty = true;
    }
  };

  const watchDirtyState = () => {
    return watchEffect(() => {
      if (ctx().isFormDirty) {
        window.onbeforeunload = () => !isDirty();
        glibevent.onbeforewindowsopen = () => !isDirty();
        glibevent.onbeforewindowsclose = () => !isDirty();
        glibevent.onbeforewindowsreload = () => !isDirty();
      } else {
        window.onbeforeunload = null;
        glibevent.onbeforewindowsopen = null;
        glibevent.onbeforewindowsclose = null;
        glibevent.onbeforewindowsreload = null;
      }
    });
  };

  return { clearDirtyState, updateDirtyState, watchDirtyState, isDirty };
}