import { vueApp } from "../../store";

export default {
  methods: {
    $componentName() { // Can be overridden
      if (this.spec) {
        return this.spec.view;
      }
    },
    $closest(name) {
      var parent = this;
      while (parent != null) {
        if (
          (Utils.type.isObject(parent.spec) || Utils.type.isObject(parent.formSpec)) &&
          parent.$componentName() == name
          // GLib.component.vueName(parent) == name
        ) {
          return parent;
        }

        parent = parent.$parent;
      }
    },
    $validation(rules) {
      var augmentedRules = rules || [];
      Utils.type.ifObject(this.spec.validation, val => {
        Utils.type.ifObject(val.required, spec => {
          augmentedRules = augmentedRules.concat([
            v => (v ? true : spec.message)
          ]);
        });
        Utils.type.ifObject(val.format, spec => {
          augmentedRules = augmentedRules.concat([
            v => {
              if (v && !v.match(spec.regex)) {
                return spec.message;
              }
              return true;
            }
          ]);
        });
      });
      return augmentedRules;
    },
    // // This can be overridden by the component to ensure that dialogs are shown within the component,
    // // which is necessary for operations requiring certain hierarchy, e.g. submitting a form from a
    // // child dialog.
    // $dialogContainer() {
    //   return document.body;
    // },
    $wsSubscribeEvents(spec) {
      if (!Utils.type.isObject(spec)) {
        return;
      }

      const topicName = spec.topic;
      const events = spec.events;
      if (!Utils.type.isString(topicName) || !Utils.type.isArray(events)) {
        return;
      }

      const ws = vueApp.webSocket;
      const channel = ws.channels[topicName];

      if (!channel) {
        console.debug(`Topic not ready: '${topicName}'`);
        setTimeout(() => {
          // Wait until $wsInitPhoenixSocket() has executed.
          this.$wsSubscribeEvents(spec);
        }, 200);
        return;
      }

      for (const eventName of events) {
        console.debug(`Registering event '${eventName}'`);
        channel.on(eventName, payload => {
          console.debug(`Received '${eventName}' event`, payload);
          Utils.ws.handleResponse(payload.onResponse, this);
        });
      }
    }
  }
};
