import { vueApp } from "../../../store";

export default {
  data: function () {
    return {
      topScrollHandler: null,
      bottomScrollHandler: null,
      nextPageUrl: null,
      prevPageUrl: null,
      request: null,
      infiniteScroll: null
    };
  },
  // computed: {
  //   bottomAnchorStyles() {
  //     return {
  //       display: this.infiniteScroll && this.nextPageUrl ? "block" : "none"
  //     };
  //   }
  // },
  methods: {
    async enableInfiniteScrollIfApplicable(spec) {
      // Make sure `page_body` has been initialized.
      await this.$nextTick()

      this.$removeViewportChangeListeners(this.topScrollHandler);
      this.$removeViewportChangeListeners(this.bottomScrollHandler);

      const prevPage = spec.prevPage || {};
      const nextPage = spec.nextPage || {};

      this.infiniteScroll =
        nextPage.autoload == "asNeeded" || prevPage.autoload == "asNeeded";
      const onScrollToBottom = spec.onScrollToBottom;
      const onScrollToTop = spec.onScrollToTop;

      // Reset state for reuse.
      this.nextPageUrl = null;
      this.prevPageUrl = null;
      if (this.infiniteScroll) {
        this.nextPageUrl = nextPage.url;
        this.prevPageUrl = prevPage.url;
      }

      // Avoid executing the scroll actions on page load.
      setTimeout(() => {
        if (this.nextPageUrl || onScrollToBottom) {
          this._registerBottomScroll(onScrollToBottom);
        }

        if (this.prevPageUrl || onScrollToTop) {
          this._registerTopScroll(onScrollToTop);
        }
      }, 1000);
    },
    _registerTopScroll(onScrollToTop) {
      this.topScrollHandler = this.$onVisibilityChange(
        this.$refs.topAnchor,
        (el, visible) => {
          if (visible) {
            this._handleTopScroll(onScrollToTop);
          }
        }
      );

      this.$addViewportChangeListeners(this.topScrollHandler);
    },
    _handleTopScroll(onScrollToTop) {
      if (this.prevPageUrl) {
        this.request = Utils.http.execute(
          { url: this.prevPageUrl },
          "GET",
          this,
          response => {
            const prevPage = response.prevPage;
            if (this.$type.isObject(prevPage)) {
              this.prevPageUrl = response.prevPage.url;
            } else {
              this.prevPageUrl = null;
            }

            var previousFirstRowIndex = null;

            for (const [
              index,
              prependedSection
            ] of response.sections.entries()) {
              const section = this.sections[index];

              Utils.type.ifArray(prependedSection.rows, rows => {
                // Get the number of rows prepended to the top section
                if (index == 0) {
                  previousFirstRowIndex = rows.length;
                }

                for (const row of rows) {
                  section.rows.unshift(row);
                }
              });

              this._updateScrollAfterPrepending(previousFirstRowIndex);
            }
            Utils.history.updatePage(vueApp.page);
          }
        );
      }

      GLib.action.execute(onScrollToTop, this);
    },
    _updateScrollAfterPrepending(previousFirstRowIndex) {
      // Scroll back to where the position was before the prepending.
      if (previousFirstRowIndex) {
        // Pre-record the height to get an accurate value
        const anchorHeight = this.$refs.topAnchor.clientHeight;

        this.$nextTick(() => {
          const prependedBottomElement = this.$refs[
            `row_0_${previousFirstRowIndex - 1}`
          ][0];

          const top =
            prependedBottomElement.offsetTop +
            prependedBottomElement.clientHeight -
            anchorHeight;

          const pageBody = Utils.history._pageBody;

          pageBody.scrollTop = top;
        });
      }
    },
    _registerBottomScroll(onScrollToBottom) {
      this.bottomScrollHandler = this.$onVisibilityChange(
        this.$refs.bottomAnchor,
        (el, visible) => {
          if (visible) {
            this._handleBottomScroll(onScrollToBottom);
          }
        }
      );

      this.$addViewportChangeListeners(this.bottomScrollHandler);
    },
    _handleBottomScroll(onScrollToBottom) {
      if (this.nextPageUrl) {
        this.request = Utils.http.execute(
          { url: this.nextPageUrl },
          "GET",
          this,
          response => {
            const nextPage = response.nextPage;
            if (this.$type.isObject(nextPage)) {
              this.nextPageUrl = response.nextPage.url;
            } else {
              this.nextPageUrl = null;
            }

            for (const section of response.sections) {
              this.sections.push(section);
            }
            Utils.history.updatePage(vueApp.page);
          }
        );
      }

      GLib.action.execute(onScrollToBottom, this);
    },
    autoloadAll(nextPage) {
      const vm = this;

      if (!this.$type.isObject(nextPage)) {
        return;
      }
      if (nextPage.autoload !== "all") {
        return;
      }

      let nextPageUrl = nextPage.url;

      this.request = Utils.http.execute(
        { url: nextPageUrl },
        "GET",
        vm,
        function (response) {
          for (const section of response.sections) {
            vm.sections.push(section);
          }
          vm.autoloadAll(response.nextPage);
        }
      );
    },
    cancelAutoloadRequest() {
      // Avoid old data coming in when this component gets reused.
      if (this.request) {
        this.request.cancel();
      }
    }
  }
};
