import { createApp, h } from "vue";
import vuetify from "./plugins/vuetify";
import App from "./app.vue";
import { settings } from "./utils/settings";
import { useTheme } from "vuetify";

// lib for deep merge
import merge from 'lodash.merge';

const Vue = createApp({
  setup() {
    const themeConfig = useTheme();
    return { themeConfig, vueApp };
  },
  created() {
    this.themeConfig.themes = merge(this.themeConfig.themes, settings.themes);
  },
  render: function () {
    return h(App, { page: this.vueApp.page });
  }
});


Vue.use(vuetify);

import { gmapPlugin } from "./plugins/gmap";
Vue.use(gmapPlugin, {
  key: import.meta.env.VITE_GMAPS_API_KEY,
  libraries: 'places'
});

// import VueAnalytics from 'vue-analytics'
// // TODO: Avoid hardcoding
// Vue.use(VueAnalytics, {
//   id: 'UA-146876849-1'
// })

import "./extensions/string.js";
import "./extensions/array.js";

// Recursive components must be global
import VerticalPanel from "./components/panels/vertical.vue";
import ResponsivePanel from "./components/panels/responsive.vue";
import Component from "./components/component.vue";
import CommonIcon from "./components/_icon.vue";
import CommonBadge from "./components/_badge.vue";
// import CommonTooltip from "./components/_tooltip.vue";
import CommonButton from "./components/_button.vue";
import CommonChip from "./components/_chip.vue";
import CommonMessage from "./components/_message.vue";
import CommonDropdownMenu from "./components/_dropdownMenu.vue";
import CommonResponsive from "./components/_responsive.vue";
import CommonTemplateMenu from "./templates/_menu.vue";
import RichButton from "./components/button.vue";
Vue.component("panels-vertical", VerticalPanel);
Vue.component("panels-responsive", ResponsivePanel);
Vue.component("common-button", CommonButton);
Vue.component("common-icon", CommonIcon);
Vue.component("common-badge", CommonBadge);
// Vue.component("common-tooltip", CommonTooltip);
Vue.component("common-chip", CommonChip);
Vue.component("common-message", CommonMessage);
Vue.component("common-dropdownMenu", CommonDropdownMenu);
Vue.component("common-responsive", CommonResponsive);
Vue.component("templates-menu", CommonTemplateMenu);

Vue.component("glib-button", RichButton);
Vue.component("glib-component", Component);
Vue.component("glib-icon", CommonIcon);
Vue.component("glib-panels-responsive", ResponsivePanel);
// Vue.component("glib-templates-menu", CommonTemplateMenu);

import genericMixin from "./components/mixins/generic.js";
Vue.mixin(genericMixin);

import eventsMixin from "./components/mixins/events.js";
Vue.mixin(eventsMixin);

import stylesMixin from "./components/mixins/styles.js";
Vue.mixin(stylesMixin);

import scrollingMixin from "./components/mixins/scrolling.js";
Vue.mixin(scrollingMixin);

import datasetMixins from "./components/mixins/dataset.js";
Vue.mixin(datasetMixins);

Vue.config.globalProperties.extension = {};
import extension from "./components/mixins/extension.js";
Vue.mixin(extension);

import longClickDirective from "./components/mixins/longClick.js";
window.longClickInstance = longClickDirective({ delay: 500, interval: 0 });
Vue.directive("longclick", window.longClickInstance);

// TODO: Deprecate
import TypeUtils from "./utils/type";
Vue.use({
  install: function (Vue, options) {
    Vue.config.globalProperties.$type = {
      isObject: TypeUtils.isObject,
      isString: TypeUtils.isString,
      isNumber: TypeUtils.isNumber,
      isArray: TypeUtils.isArray,

      ifObject: TypeUtils.ifObject,
      ifString: TypeUtils.ifString,
      ifNumber: TypeUtils.ifNumber,
      ifArray: TypeUtils.ifArray
    };
  }
});

import Utils from "./utils/helper";
window.Utils = Utils;

import Framework from "./utils/public";
window.GLib = Framework;

import driverCustomBehavior from "./plugins/driverCustomBehavior";
Vue.use(driverCustomBehavior);

import updatableComponent from "./plugins/updatableComponent";
Vue.use(updatableComponent);

// TODO: https://vuejs.org/v2/guide/components-dynamic-async.html#Async-Components
// Vue.component('async-webpack-example', function (resolve) {
//   // This special require syntax will instruct Webpack to
//   // automatically split your built code into bundles which
//   // are loaded over Ajax requests.
//   require(['./my-async-component'], resolve)
// })


import { vueApp } from "./store";

document.addEventListener("DOMContentLoaded", () => {
  Vue.mount("#app");
});

export { Vue, settings, vueApp };

// The above code uses Vue without the compiler, which means you cannot
// use Vue to target elements in your existing html templates. You would
// need to always use single file components.
// To be able to target elements in your existing html/erb templates,
// comment out the above code and uncomment the below
// Add <%= javascript_pack_tag 'hello_vue' %> to your layout
// Then add this markup to your html template:
//
// <div id='hello'>
//   {{message}}
//   <app></app>
// </div>

// import Vue from 'vue/dist/vue.esm'
// import App from '../app.vue'
//
// document.addEventListener('DOMContentLoaded', () => {
//   const app = new Vue({
//     el: '#hello',
//     data: {
//       message: "Can you say hello?"
//     },
//     components: { App }
//   })
// })
