import { nextTick } from "vue";
import { vueApp } from "../store";


export default class {
  static get _registry() {
    return vueApp.registeredComponents;
  }

  static findById(id) {
    return vueApp.registeredComponents[id];
  }

  static register(id, component) {
    // if (this._registry[id]) {
    //   console.warn(
    //     "Duplicate component ID:",
    //     id,
    //     "Existing:",
    //     this.vueName(this._registry[id]),
    //     "New:",
    //     this.vueName(component)
    //   );
    // }

    this._registry[id] = component;
  }

  static deregister(id, component) {
    // As the user navigates to a new page, there may be a component with the same ID in the next page,
    // that is not reused from the component in the previous page. When this happens, the new
    // component will get registered before the old component gets deregistered, in which case we
    // should skip the deregistering or else we'd accidentally deregister the new component.
    if (this._registry[id] == component) {
      console.debug("Deregistering component with ID", id);
      delete this._registry[id];
    }
  }

  static vueName(component) {
    // _componentTag not found
    // return component.$options._componentTag;
    if (!component.name && component.spec) {
      if (component.spec.view == 'panels/fullPageForm') {
        component.name = 'panels-form';
      } else {
        component.name = (component.spec.view || '').replace('/', '-').replace('-v1', '');
      }
    }

    return component.name;
  }

  static async preserveScroll(promise) {
    const pageBody = this.$pageBody;
    const originalTop = pageBody.scrollTop;
    const originalLeft = pageBody.scrollLeft;

    await promise;

    nextTick(() => {
      pageBody.scrollTop = originalTop;
      pageBody.scrollLeft = originalLeft;
    });
  }

  static get $pageBody() {
    return document.getElementById("page_body");
  }
}
