import { ctx } from "../store";

export default class {
  // Execution flows:
  // Button -> Button.onClick() -> Utils.http.execute() -- doesn't save autofill values
  // Button -> Button.onClick() -> form.submit() -- doesn't execute Form.onSubmit(), so cannot call Utils.http.execute() there
  // Submit -> Form.onSubmit() -> Utils.http.execute() -- save autofill values and auto-submit when ENTER is pressed
  static submitData(form, component, overrideUrl = null, overrideMethod = null) {
    // Analogous to Rails' form_with's `local: true`
    if (form.dataset.local) {
      if (overrideUrl) {
        component.$data.url = overrideUrl
      }

      // Prevent onUnload dirty prompt.
      // Utils.http.clearDirtyState();

      // Use nextTick to wait for overrideUrl get picked up.
      component.$nextTick(() => {
        ctx().isFormDirty = false;
        form.submit();
      });
    } else {
      const url = overrideUrl ? overrideUrl : form.getAttribute("action");
      const formData = new FormData(form);
      const method = overrideMethod ? overrideMethod : form.getAttribute("method");

      if (method.toUpperCase() === "GET") {
        // Don't include this token in GET requests.
        formData.delete("authenticity_token")
        const data = {
          url: Utils.url.appendParams(url, formData)
        };
        Utils.http.notifyFormSubmitted();
        Utils.http.load(data, component, false);
      } else {
        const data = {
          url: url,
          formData: formData
        };
        Utils.http.execute(data, method, component, response => {
          Utils.http.notifyFormSubmitted();
          GLib.action.handleResponse(response, component);
        });
      }
    }
  }
}
