import { nextTick } from "vue";
import { vueApp } from "../store";

export default class {
  static saveInitialContent(content) {
    // History only handles document's scroll position. It doesn't handle our body div's position
    this.bodyScrollTops = [];
    // This differs from window.history.length in that this only counts internal navigation
    this.navigationCount = 0;
    // Set initial state so we can go back to it
    window.history.replaceState(this._data(content), "", window.location.href);
  }

  static get _pageBody() {
    return document.getElementById("page_body");
  }

  static get _innerBody() {
    // TODO: make this support any panelId
    return document.getElementById("inner_body");
  }

  static _data(content) {
    return { content: JSON.parse(JSON.stringify(content)), navigationCount: this.navigationCount };
  }

  static pushPage(content, url) {
    // Save scroll position of the current page when navigating to the next page
    this.bodyScrollTops[this.navigationCount] = this._pageBody.scrollTop;

    this.navigationCount++;

    // Save data of the next page (page getting pushed)
    window.history.pushState(this._data(content), "", url);
  }

  static resetScroll() {
    this._pageBody.scrollTop = 0;
  }

  static restoreOnBackOrForward() {
    const vm = this;
    window.onpopstate = event => {
      // TODO: Ideally display a prompt when dirty
      // if (Utils.http.proceedEvenWhenDirty()) {
      //   event.preventDefault();
      //   history.go(1);
      //   return false;
      // }

      vueApp.isFormDirty = false

      // Save scroll position of the current page when navigating through back/forward button
      this.bodyScrollTops[this.navigationCount] = this._pageBody.scrollTop;

      const data = event.state;
      data.content.__poppedState = true;
      // 

      vueApp.page = data.content
      vm.navigationCount = data.navigationCount;

      nextTick(() => {
        const scrollTop = this.bodyScrollTops[this.navigationCount];
        this._pageBody.scrollTop = scrollTop;
      });
    };
  }

  static isPoppedState(page) {
    return page.__poppedState == true;
  }

  static updatePage(page, url = window.location.href) {
    window.history.replaceState(this._data(page), "", url);
  }

  static back() {
    if (this.navigationCount > 0) {
      window.history.back();
      return true;
    } else {
      console.debug("Reached beginning of navigation stack");
      return false;
    }
  }
}
